@@ -21,7 +21,6 @@
 
 struct fsl_xcvr_soc_data {
 	const char *fw_name;
-	bool spdif_only;
 };
 
 struct fsl_xcvr {
@@ -203,13 +202,10 @@ static int fsl_xcvr_mode_put(struct snd_kcontrol *kcontrol,
 
 	xcvr->mode = snd_soc_enum_item_to_val(e, item[0]);
 
-	down_read(&card->snd_card->controls_rwsem);
 	fsl_xcvr_activate_ctl(dai, fsl_xcvr_arc_mode_kctl.name,
 			      (xcvr->mode == FSL_XCVR_MODE_ARC));
 	fsl_xcvr_activate_ctl(dai, fsl_xcvr_earc_capds_kctl.name,
 			      (xcvr->mode == FSL_XCVR_MODE_EARC));
-	up_read(&card->snd_card->controls_rwsem);
-
 	/* Allow playback for SPDIF only */
 	rtd = snd_soc_get_pcm_runtime(card, card->dai_link);
 	rtd->pcm->streams[SNDRV_PCM_STREAM_PLAYBACK].substream_count =
@@ -265,9 +261,6 @@ static int fsl_xcvr_en_phy_pll(struct fsl_xcvr *xcvr, u32 freq, bool tx)
 	u32 i, div = 0, log2;
 	int ret;
 
-	if (xcvr->soc_data->spdif_only)
-		return 0;
-
 	for (i = 0; i < ARRAY_SIZE(fsl_xcvr_pll_cfg); i++) {
 		if (fsl_xcvr_pll_cfg[i].fout % freq == 0) {
 			div = fsl_xcvr_pll_cfg[i].fout / freq;
@@ -360,7 +353,6 @@ static int fsl_xcvr_en_aud_pll(struct fsl_xcvr *xcvr, u32 freq)
 	struct device *dev = &xcvr->pdev->dev;
 	int ret;
 
-	freq = xcvr->soc_data->spdif_only ? freq / 10 : freq;
 	clk_disable_unprepare(xcvr->phy_clk);
 	ret = clk_set_rate(xcvr->phy_clk, freq);
 	if (ret < 0) {
@@ -373,8 +365,6 @@ static int fsl_xcvr_en_aud_pll(struct fsl_xcvr *xcvr, u32 freq)
 		return ret;
 	}
 
-	if (xcvr->soc_data->spdif_only)
-		return 0;
 	/* Release AI interface from reset */
 	ret = regmap_write(xcvr->regmap, FSL_XCVR_PHY_AI_CTRL_SET,
 			   FSL_XCVR_PHY_AI_CTRL_AI_RESETN);
@@ -557,12 +547,10 @@ static int fsl_xcvr_startup(struct snd_pcm_substream *substream,
 
 	xcvr->streams |= BIT(substream->stream);
 
-	if (!xcvr->soc_data->spdif_only) {
-		/* Disable XCVR controls if there is stream started */
-		fsl_xcvr_activate_ctl(dai, fsl_xcvr_mode_kctl.name, false);
-		fsl_xcvr_activate_ctl(dai, fsl_xcvr_arc_mode_kctl.name, false);
-		fsl_xcvr_activate_ctl(dai, fsl_xcvr_earc_capds_kctl.name, false);
-	}
+	/* Disable XCVR controls if there is stream started */
+	fsl_xcvr_activate_ctl(dai, fsl_xcvr_mode_kctl.name, false);
+	fsl_xcvr_activate_ctl(dai, fsl_xcvr_arc_mode_kctl.name, false);
+	fsl_xcvr_activate_ctl(dai, fsl_xcvr_earc_capds_kctl.name, false);
 
 	return 0;
 }
@@ -579,13 +567,12 @@ static void fsl_xcvr_shutdown(struct snd_pcm_substream *substream,
 
 	/* Enable XCVR controls if there is no stream started */
 	if (!xcvr->streams) {
-		if (!xcvr->soc_data->spdif_only) {
-			fsl_xcvr_activate_ctl(dai, fsl_xcvr_mode_kctl.name, true);
-			fsl_xcvr_activate_ctl(dai, fsl_xcvr_arc_mode_kctl.name,
-						(xcvr->mode == FSL_XCVR_MODE_ARC));
-			fsl_xcvr_activate_ctl(dai, fsl_xcvr_earc_capds_kctl.name,
-						(xcvr->mode == FSL_XCVR_MODE_EARC));
-		}
+		fsl_xcvr_activate_ctl(dai, fsl_xcvr_mode_kctl.name, true);
+		fsl_xcvr_activate_ctl(dai, fsl_xcvr_arc_mode_kctl.name,
+				      (xcvr->mode == FSL_XCVR_MODE_ARC));
+		fsl_xcvr_activate_ctl(dai, fsl_xcvr_earc_capds_kctl.name,
+				      (xcvr->mode == FSL_XCVR_MODE_EARC));
+
 		ret = regmap_update_bits(xcvr->regmap, FSL_XCVR_EXT_IER0,
 					 FSL_XCVR_IRQ_EARC_ALL, 0);
 		if (ret < 0) {
@@ -686,10 +673,7 @@ static int fsl_xcvr_trigger(struct snd_pcm_substream *substream, int cmd,
 					dev_err(dai->dev, "Failed to stop DATA_TX: %d\n", ret);
 					return ret;
 				}
-				if (xcvr->soc_data->spdif_only)
-					break;
-				else
-					fallthrough;
+				fallthrough;
 			case FSL_XCVR_MODE_EARC:
 				/* clear ISR_CMDC_TX_EN, W1C */
 				ret = regmap_write(xcvr->regmap,
@@ -880,26 +864,15 @@ static struct snd_kcontrol_new fsl_xcvr_tx_ctls[] = {
 	},
 };
 
-static const struct snd_soc_dai_ops fsl_xcvr_dai_ops = {
-	.prepare = fsl_xcvr_prepare,
-	.startup = fsl_xcvr_startup,
-	.shutdown = fsl_xcvr_shutdown,
-	.trigger = fsl_xcvr_trigger,
-};
-
 static int fsl_xcvr_dai_probe(struct snd_soc_dai *dai)
 {
 	struct fsl_xcvr *xcvr = snd_soc_dai_get_drvdata(dai);
 
 	snd_soc_dai_init_dma_data(dai, &xcvr->dma_prms_tx, &xcvr->dma_prms_rx);
 
-	if (xcvr->soc_data->spdif_only)
-		xcvr->mode = FSL_XCVR_MODE_SPDIF;
-	else {
-		snd_soc_add_dai_controls(dai, &fsl_xcvr_mode_kctl, 1);
-		snd_soc_add_dai_controls(dai, &fsl_xcvr_arc_mode_kctl, 1);
-		snd_soc_add_dai_controls(dai, &fsl_xcvr_earc_capds_kctl, 1);
-	}
+	snd_soc_add_dai_controls(dai, &fsl_xcvr_mode_kctl, 1);
+	snd_soc_add_dai_controls(dai, &fsl_xcvr_arc_mode_kctl, 1);
+	snd_soc_add_dai_controls(dai, &fsl_xcvr_earc_capds_kctl, 1);
 	snd_soc_add_dai_controls(dai, fsl_xcvr_tx_ctls,
 				 ARRAY_SIZE(fsl_xcvr_tx_ctls));
 	snd_soc_add_dai_controls(dai, fsl_xcvr_rx_ctls,
@@ -907,8 +880,15 @@ static int fsl_xcvr_dai_probe(struct snd_soc_dai *dai)
 	return 0;
 }
 
+static const struct snd_soc_dai_ops fsl_xcvr_dai_ops = {
+	.probe		= fsl_xcvr_dai_probe,
+	.prepare	= fsl_xcvr_prepare,
+	.startup	= fsl_xcvr_startup,
+	.shutdown	= fsl_xcvr_shutdown,
+	.trigger	= fsl_xcvr_trigger,
+};
+
 static struct snd_soc_dai_driver fsl_xcvr_dai = {
-	.probe  = fsl_xcvr_dai_probe,
 	.ops = &fsl_xcvr_dai_ops,
 	.playback = {
 		.stream_name = "CPU-Playback",
@@ -931,7 +911,8 @@ static struct snd_soc_dai_driver fsl_xcvr_dai = {
 };
 
 static const struct snd_soc_component_driver fsl_xcvr_comp = {
-	.name = "fsl-xcvr-dai",
+	.name			= "fsl-xcvr-dai",
+	.legacy_dai_naming	= 1,
 };
 
 static const struct reg_default fsl_xcvr_reg_defaults[] = {
@@ -949,19 +930,10 @@ static const struct reg_default fsl_xcvr_reg_defaults[] = {
 	{ FSL_XCVR_ISR_SET,	0x00000000 },
 	{ FSL_XCVR_ISR_CLR,	0x00000000 },
 	{ FSL_XCVR_ISR_TOG,	0x00000000 },
-	{ FSL_XCVR_CLK_CTRL,	0x0000018F },
-	{ FSL_XCVR_RX_DPTH_CTRL,	0x00040CC1 },
-	{ FSL_XCVR_RX_DPTH_CTRL_SET,	0x00040CC1 },
-	{ FSL_XCVR_RX_DPTH_CTRL_CLR,	0x00040CC1 },
-	{ FSL_XCVR_RX_DPTH_CTRL_TOG,	0x00040CC1 },
-	{ FSL_XCVR_RX_DPTH_CNTR_CTRL,	0x00000000 },
-	{ FSL_XCVR_RX_DPTH_CNTR_CTRL_SET, 0x00000000 },
-	{ FSL_XCVR_RX_DPTH_CNTR_CTRL_CLR, 0x00000000 },
-	{ FSL_XCVR_RX_DPTH_CNTR_CTRL_TOG, 0x00000000 },
-	{ FSL_XCVR_RX_DPTH_TSCR, 0x00000000 },
-	{ FSL_XCVR_RX_DPTH_BCR,  0x00000000 },
-	{ FSL_XCVR_RX_DPTH_BCTR, 0x00000000 },
-	{ FSL_XCVR_RX_DPTH_BCRR, 0x00000000 },
+	{ FSL_XCVR_RX_DPTH_CTRL,	0x00002C89 },
+	{ FSL_XCVR_RX_DPTH_CTRL_SET,	0x00002C89 },
+	{ FSL_XCVR_RX_DPTH_CTRL_CLR,	0x00002C89 },
+	{ FSL_XCVR_RX_DPTH_CTRL_TOG,	0x00002C89 },
 	{ FSL_XCVR_TX_DPTH_CTRL,	0x00000000 },
 	{ FSL_XCVR_TX_DPTH_CTRL_SET,	0x00000000 },
 	{ FSL_XCVR_TX_DPTH_CTRL_CLR,	0x00000000 },
@@ -972,26 +944,12 @@ static const struct reg_default fsl_xcvr_reg_defaults[] = {
 	{ FSL_XCVR_TX_CS_DATA_3,	0x00000000 },
 	{ FSL_XCVR_TX_CS_DATA_4,	0x00000000 },
 	{ FSL_XCVR_TX_CS_DATA_5,	0x00000000 },
-	{ FSL_XCVR_TX_DPTH_CNTR_CTRL,	0x00000000 },
-	{ FSL_XCVR_TX_DPTH_CNTR_CTRL_SET, 0x00000000 },
-	{ FSL_XCVR_TX_DPTH_CNTR_CTRL_CLR, 0x00000000 },
-	{ FSL_XCVR_TX_DPTH_CNTR_CTRL_TOG, 0x00000000 },
-	{ FSL_XCVR_TX_DPTH_TSCR, 0x00000000 },
-	{ FSL_XCVR_TX_DPTH_BCR,	 0x00000000 },
-	{ FSL_XCVR_TX_DPTH_BCTR, 0x00000000 },
-	{ FSL_XCVR_TX_DPTH_BCRR, 0x00000000 },
 	{ FSL_XCVR_DEBUG_REG_0,		0x00000000 },
 	{ FSL_XCVR_DEBUG_REG_1,		0x00000000 },
 };
 
 static bool fsl_xcvr_readable_reg(struct device *dev, unsigned int reg)
 {
-	struct fsl_xcvr *xcvr = dev_get_drvdata(dev);
-
-	if (xcvr->soc_data->spdif_only)
-		if ((reg >= FSL_XCVR_IER && reg <= FSL_XCVR_PHY_AI_RDATA) ||
-		    reg > FSL_XCVR_TX_DPTH_BCRR)
-			return false;
 	switch (reg) {
 	case FSL_XCVR_VERSION:
 	case FSL_XCVR_EXT_CTRL:
@@ -1017,20 +975,6 @@ static bool fsl_xcvr_readable_reg(struct device *dev, unsigned int reg)
 	case FSL_XCVR_RX_DPTH_CTRL_SET:
 	case FSL_XCVR_RX_DPTH_CTRL_CLR:
 	case FSL_XCVR_RX_DPTH_CTRL_TOG:
-	case FSL_XCVR_RX_CS_DATA_0:
-	case FSL_XCVR_RX_CS_DATA_1:
-	case FSL_XCVR_RX_CS_DATA_2:
-	case FSL_XCVR_RX_CS_DATA_3:
-	case FSL_XCVR_RX_CS_DATA_4:
-	case FSL_XCVR_RX_CS_DATA_5:
-	case FSL_XCVR_RX_DPTH_CNTR_CTRL:
-	case FSL_XCVR_RX_DPTH_CNTR_CTRL_SET:
-	case FSL_XCVR_RX_DPTH_CNTR_CTRL_CLR:
-	case FSL_XCVR_RX_DPTH_CNTR_CTRL_TOG:
-	case FSL_XCVR_RX_DPTH_TSCR:
-	case FSL_XCVR_RX_DPTH_BCR:
-	case FSL_XCVR_RX_DPTH_BCTR:
-	case FSL_XCVR_RX_DPTH_BCRR:
 	case FSL_XCVR_TX_DPTH_CTRL:
 	case FSL_XCVR_TX_DPTH_CTRL_SET:
 	case FSL_XCVR_TX_DPTH_CTRL_CLR:
@@ -1041,14 +985,6 @@ static bool fsl_xcvr_readable_reg(struct device *dev, unsigned int reg)
 	case FSL_XCVR_TX_CS_DATA_3:
 	case FSL_XCVR_TX_CS_DATA_4:
 	case FSL_XCVR_TX_CS_DATA_5:
-	case FSL_XCVR_TX_DPTH_CNTR_CTRL:
-	case FSL_XCVR_TX_DPTH_CNTR_CTRL_SET:
-	case FSL_XCVR_TX_DPTH_CNTR_CTRL_CLR:
-	case FSL_XCVR_TX_DPTH_CNTR_CTRL_TOG:
-	case FSL_XCVR_TX_DPTH_TSCR:
-	case FSL_XCVR_TX_DPTH_BCR:
-	case FSL_XCVR_TX_DPTH_BCTR:
-	case FSL_XCVR_TX_DPTH_BCRR:
 	case FSL_XCVR_DEBUG_REG_0:
 	case FSL_XCVR_DEBUG_REG_1:
 		return true;
@@ -1059,11 +995,6 @@ static bool fsl_xcvr_readable_reg(struct device *dev, unsigned int reg)
 
 static bool fsl_xcvr_writeable_reg(struct device *dev, unsigned int reg)
 {
-	struct fsl_xcvr *xcvr = dev_get_drvdata(dev);
-
-	if (xcvr->soc_data->spdif_only)
-		if (reg >= FSL_XCVR_IER && reg <= FSL_XCVR_PHY_AI_RDATA)
-			return false;
 	switch (reg) {
 	case FSL_XCVR_EXT_CTRL:
 	case FSL_XCVR_EXT_IER0:
@@ -1086,10 +1017,6 @@ static bool fsl_xcvr_writeable_reg(struct device *dev, unsigned int reg)
 	case FSL_XCVR_RX_DPTH_CTRL_SET:
 	case FSL_XCVR_RX_DPTH_CTRL_CLR:
 	case FSL_XCVR_RX_DPTH_CTRL_TOG:
-	case FSL_XCVR_RX_DPTH_CNTR_CTRL:
-	case FSL_XCVR_RX_DPTH_CNTR_CTRL_SET:
-	case FSL_XCVR_RX_DPTH_CNTR_CTRL_CLR:
-	case FSL_XCVR_RX_DPTH_CNTR_CTRL_TOG:
 	case FSL_XCVR_TX_DPTH_CTRL_SET:
 	case FSL_XCVR_TX_DPTH_CTRL_CLR:
 	case FSL_XCVR_TX_DPTH_CTRL_TOG:
@@ -1099,10 +1026,6 @@ static bool fsl_xcvr_writeable_reg(struct device *dev, unsigned int reg)
 	case FSL_XCVR_TX_CS_DATA_3:
 	case FSL_XCVR_TX_CS_DATA_4:
 	case FSL_XCVR_TX_CS_DATA_5:
-	case FSL_XCVR_TX_DPTH_CNTR_CTRL:
-	case FSL_XCVR_TX_DPTH_CNTR_CTRL_SET:
-	case FSL_XCVR_TX_DPTH_CNTR_CTRL_CLR:
-	case FSL_XCVR_TX_DPTH_CNTR_CTRL_TOG:
 		return true;
 	default:
 		return false;
@@ -1140,54 +1063,32 @@ static irqreturn_t irq0_isr(int irq, void *devid)
 	if (isr & FSL_XCVR_IRQ_NEW_CS) {
 		dev_dbg(dev, "Received new CS block\n");
 		isr_clr |= FSL_XCVR_IRQ_NEW_CS;
-		if (!xcvr->soc_data->spdif_only) {
-			/* Data RAM is 4KiB, last two pages: 8 and 9. Select page 8. */
-			regmap_update_bits(xcvr->regmap, FSL_XCVR_EXT_CTRL,
-					   FSL_XCVR_EXT_CTRL_PAGE_MASK,
-					   FSL_XCVR_EXT_CTRL_PAGE(8));
+		/* Data RAM is 4KiB, last two pages: 8 and 9. Select page 8. */
+		regmap_update_bits(xcvr->regmap, FSL_XCVR_EXT_CTRL,
+				   FSL_XCVR_EXT_CTRL_PAGE_MASK,
+				   FSL_XCVR_EXT_CTRL_PAGE(8));
 
-			/* Find updated CS buffer */
-			reg_ctrl = xcvr->ram_addr + FSL_XCVR_RX_CS_CTRL_0;
-			reg_buff = xcvr->ram_addr + FSL_XCVR_RX_CS_BUFF_0;
+		/* Find updated CS buffer */
+		reg_ctrl = xcvr->ram_addr + FSL_XCVR_RX_CS_CTRL_0;
+		reg_buff = xcvr->ram_addr + FSL_XCVR_RX_CS_BUFF_0;
+		memcpy_fromio(&val, reg_ctrl, sizeof(val));
+		if (!val) {
+			reg_ctrl = xcvr->ram_addr + FSL_XCVR_RX_CS_CTRL_1;
+			reg_buff = xcvr->ram_addr + FSL_XCVR_RX_CS_BUFF_1;
 			memcpy_fromio(&val, reg_ctrl, sizeof(val));
-			if (!val) {
-				reg_ctrl = xcvr->ram_addr + FSL_XCVR_RX_CS_CTRL_1;
-				reg_buff = xcvr->ram_addr + FSL_XCVR_RX_CS_BUFF_1;
-				memcpy_fromio(&val, reg_ctrl, sizeof(val));
-			}
+		}
 
-			if (val) {
-				/* copy CS buffer */
-				memcpy_fromio(&xcvr->rx_iec958.status, reg_buff,
-					      sizeof(xcvr->rx_iec958.status));
-				for (i = 0; i < 6; i++) {
-					val = *(u32 *)(xcvr->rx_iec958.status + i*4);
-					*(u32 *)(xcvr->rx_iec958.status + i*4) =
-						bitrev32(val);
-				}
-				/* clear CS control register */
-				writel_relaxed(0, reg_ctrl);
-			}
-		} else {
-			regmap_read(xcvr->regmap, FSL_XCVR_RX_CS_DATA_0,
-				    (u32 *)&xcvr->rx_iec958.status[0]);
-			regmap_read(xcvr->regmap, FSL_XCVR_RX_CS_DATA_1,
-				    (u32 *)&xcvr->rx_iec958.status[4]);
-			regmap_read(xcvr->regmap, FSL_XCVR_RX_CS_DATA_2,
-				    (u32 *)&xcvr->rx_iec958.status[8]);
-			regmap_read(xcvr->regmap, FSL_XCVR_RX_CS_DATA_3,
-				    (u32 *)&xcvr->rx_iec958.status[12]);
-			regmap_read(xcvr->regmap, FSL_XCVR_RX_CS_DATA_4,
-				    (u32 *)&xcvr->rx_iec958.status[16]);
-			regmap_read(xcvr->regmap, FSL_XCVR_RX_CS_DATA_5,
-				    (u32 *)&xcvr->rx_iec958.status[20]);
+		if (val) {
+			/* copy CS buffer */
+			memcpy_fromio(&xcvr->rx_iec958.status, reg_buff,
+				      sizeof(xcvr->rx_iec958.status));
 			for (i = 0; i < 6; i++) {
-				val = *(u32 *)(xcvr->rx_iec958.status + i * 4);
-				*(u32 *)(xcvr->rx_iec958.status + i * 4) =
+				val = *(u32 *)(xcvr->rx_iec958.status + i*4);
+				*(u32 *)(xcvr->rx_iec958.status + i*4) =
 					bitrev32(val);
 			}
-			regmap_set_bits(xcvr->regmap, FSL_XCVR_RX_DPTH_CTRL,
-					FSL_XCVR_RX_DPTH_CTRL_CSA);
+			/* clear CS control register */
+			memset_io(reg_ctrl, 0, sizeof(val));
 		}
 	}
 	if (isr & FSL_XCVR_IRQ_NEW_UD) {
@@ -1227,13 +1128,8 @@ static const struct fsl_xcvr_soc_data fsl_xcvr_imx8mp_data = {
 	.fw_name = "/*(DEBLOBBED)*/",
 };
 
-static const struct fsl_xcvr_soc_data fsl_xcvr_imx93_data = {
-	.spdif_only = true,
-};
-
 static const struct of_device_id fsl_xcvr_dt_ids[] = {
 	{ .compatible = "fsl,imx8mp-xcvr", .data = &fsl_xcvr_imx8mp_data },
-	{ .compatible = "fsl,imx93-xcvr", .data = &fsl_xcvr_imx93_data},
 	{ /* sentinel */ }
 };
 MODULE_DEVICE_TABLE(of, fsl_xcvr_dt_ids);
@@ -1293,7 +1189,7 @@ static int fsl_xcvr_probe(struct platform_device *pdev)
 		return PTR_ERR(xcvr->regmap);
 	}
 
-	xcvr->reset = devm_reset_control_get_optional_exclusive(dev, NULL);
+	xcvr->reset = devm_reset_control_get_exclusive(dev, NULL);
 	if (IS_ERR(xcvr->reset)) {
 		dev_err(dev, "failed to get XCVR reset control\n");
 		return PTR_ERR(xcvr->reset);
@@ -1333,6 +1229,7 @@ static int fsl_xcvr_probe(struct platform_device *pdev)
 	 */
 	ret = devm_snd_dmaengine_pcm_register(dev, NULL, 0);
 	if (ret) {
+		pm_runtime_disable(dev);
 		dev_err(dev, "failed to pcm register\n");
 		return ret;
 	}
@@ -1340,6 +1237,7 @@ static int fsl_xcvr_probe(struct platform_device *pdev)
 	ret = devm_snd_soc_register_component(dev, &fsl_xcvr_comp,
 					      &fsl_xcvr_dai, 1);
 	if (ret) {
+		pm_runtime_disable(dev);
 		dev_err(dev, "failed to register component %s\n",
 			fsl_xcvr_comp.name);
 	}
@@ -1347,6 +1245,12 @@ static int fsl_xcvr_probe(struct platform_device *pdev)
 	return ret;
 }
 
+static int fsl_xcvr_remove(struct platform_device *pdev)
+{
+	pm_runtime_disable(&pdev->dev);
+	return 0;
+}
+
 static __maybe_unused int fsl_xcvr_runtime_suspend(struct device *dev)
 {
 	struct fsl_xcvr *xcvr = dev_get_drvdata(dev);
@@ -1362,14 +1266,12 @@ static __maybe_unused int fsl_xcvr_runtime_suspend(struct device *dev)
 	if (ret < 0)
 		dev_err(dev, "Failed to clear IER0: %d\n", ret);
 
-	if (!xcvr->soc_data->spdif_only) {
-		/* Assert M0+ reset */
-		ret = regmap_update_bits(xcvr->regmap, FSL_XCVR_EXT_CTRL,
-					FSL_XCVR_EXT_CTRL_CORE_RESET,
-					FSL_XCVR_EXT_CTRL_CORE_RESET);
-		if (ret < 0)
-			dev_err(dev, "Failed to assert M0+ core: %d\n", ret);
-	}
+	/* Assert M0+ reset */
+	ret = regmap_update_bits(xcvr->regmap, FSL_XCVR_EXT_CTRL,
+				 FSL_XCVR_EXT_CTRL_CORE_RESET,
+				 FSL_XCVR_EXT_CTRL_CORE_RESET);
+	if (ret < 0)
+		dev_err(dev, "Failed to assert M0+ core: %d\n", ret);
 
 	regcache_cache_only(xcvr->regmap, true);
 
@@ -1425,9 +1327,6 @@ static __maybe_unused int fsl_xcvr_runtime_resume(struct device *dev)
 		goto stop_spba_clk;
 	}
 
-	if (xcvr->soc_data->spdif_only)
-		return 0;
-
 	ret = reset_control_deassert(xcvr->reset);
 	if (ret) {
 		dev_err(dev, "failed to deassert M0+ reset.\n");
@@ -1480,6 +1379,7 @@ static struct platform_driver fsl_xcvr_driver = {
 		.pm = &fsl_xcvr_pm_ops,
 		.of_match_table = fsl_xcvr_dt_ids,
 	},
+	.remove = fsl_xcvr_remove,
 };
 module_platform_driver(fsl_xcvr_driver);
 
